"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createWebSocketServer = createWebSocketServer;
const events = __importStar(require("events"));
const koa_compose_1 = __importDefault(require("koa-compose"));
const ws_1 = require("ws");
const accesslog_1 = require("./middlewares/accesslog");
const async_context_1 = require("./middlewares/async-context");
const catch_error_1 = require("./middlewares/catch-error");
const concurrency_limit_1 = require("./middlewares/concurrency-limit");
const context_1 = require("./middlewares/context");
const eventid_1 = require("./middlewares/eventid");
const healthz_1 = require("./middlewares/healthz");
const timeout_1 = require("./middlewares/timeout");
function createWebSocketServer(httpServer, koaApp, userFunction) {
    const wss = new ws_1.WebSocketServer({
        noServer: true,
        perMessageDeflate: {
            zlibDeflateOptions: {
                // See zlib defaults.
                chunkSize: 1024,
                memLevel: 7,
                level: 3
            },
            zlibInflateOptions: {
                chunkSize: 10 * 1024
            },
            // Other options settable:
            clientNoContextTakeover: true, // Defaults to negotiated value.
            serverNoContextTakeover: true, // Defaults to negotiated value.
            serverMaxWindowBits: 10, // Defaults to negotiated value.
            // Below options specified as default values.
            concurrencyLimit: 10, // Limits zlib concurrency for perf.
            threshold: 1024 // Size (in bytes) below which messages
            // should not be compressed if context takeover is disabled.
        }
    });
    // wss.on('error', err => {})
    // wss.on('connection', () => {})
    // wss.on('headers', () => {})
    // wss.on('close', () => {})
    // wss.on('listening', () => {})
    httpServer.on('upgrade', async (request, socket, head) => {
        console.log('upgrade...');
        if (typeof userFunction.handleUpgrade !== 'function') {
            socket.destroy();
            return;
        }
        /* eslint-disable-next-line */
        const ctx = koaApp.createContext(request, {});
        const fn = (0, koa_compose_1.default)([
            (0, healthz_1.healthzMiddleware)(),
            (0, eventid_1.eventidMiddleware)(),
            (0, async_context_1.asyncContextMiddleware)(),
            (0, accesslog_1.accessLogMiddleware)({
                logHeaderBody: false,
                logEventContext: false
            }),
            (0, catch_error_1.catchErrorMiddleware)(),
            (0, timeout_1.timeoutMiddleware)(10000),
            (0, concurrency_limit_1.concurrencyLimitMiddleware)(1000),
            (0, context_1.contextInjectionMiddleware)()
        ]);
        try {
            await fn(ctx, () => {
                return new Promise((resolve, reject) => {
                    if (typeof userFunction.handleUpgrade !== 'function') {
                        socket.destroy();
                        return resolve(undefined);
                    }
                    const context = {
                        eventID: ctx.state.eventID,
                        eventType: 'wesocket.upgrade',
                        timestamp: new Date().toISOString(),
                        tcbContext: ctx.state.tcbContext,
                        wechatContext: ctx.state.wechatContext,
                        httpContext: {
                            url: request.url,
                            httpMethod: request.method,
                            headers: request.headers
                        }
                    };
                    try {
                        userFunction.handleUpgrade(context).then((result) => {
                            console.log('result: ', result);
                            if (result.allowUpgrade) {
                                wss.handleUpgrade(request, socket, head, async (ws) => {
                                    const context = {
                                        eventID: ctx.state.eventID,
                                        eventType: 'wesocket.connection',
                                        timestamp: new Date().toISOString(),
                                        tcbContext: ctx.state.tcbContext,
                                        wechatContext: ctx.state.wechatContext,
                                        httpContext: {
                                            url: request.url,
                                            httpMethod: request.method,
                                            headers: request.headers
                                        }
                                    };
                                    context.ws = new SampleWebSocket(ws, context);
                                    try {
                                        await userFunction(null, { ...context });
                                    }
                                    catch (e) {
                                        // ignore e
                                    }
                                    resolve(undefined);
                                    // ws.on('close', console.log)
                                    // ws.on('error', console.log)
                                    // ws.on('upgrade', console.log)
                                    // ws.on('open', console.log)
                                    // ws.on('ping', console.log)
                                    // ws.on('pong', console.log)
                                    // ws.on('unexpected-response', console.log)
                                    // ws.on('message', console.log)
                                });
                            }
                        }).catch(reject);
                    }
                    catch (e) {
                        socket.destroy();
                        return reject(e);
                    }
                });
            });
        }
        catch (e) {
            console.error(e, 'eee');
        }
    });
}
class SampleWebSocket extends events.EventEmitter {
    ws;
    ctx;
    constructor(ws, ctx) {
        super();
        this.ws = ws;
        this.ctx = ctx;
        const baseEvent = {
            eventId: this.ctx.eventID
        };
        this.ws.onopen = (openEvent) => {
            this.emit('onen', {
                ...baseEvent,
                type: openEvent.type
            });
        };
        this.ws.onerror = (errorEvent) => {
            this.emit('error', {
                ...baseEvent,
                type: errorEvent.type,
                message: errorEvent.message,
                error: errorEvent.error
            });
        };
        this.ws.onclose = (closeEvent) => {
            this.emit('close', {
                ...baseEvent,
                type: closeEvent.type,
                code: closeEvent.code,
                reason: closeEvent.reason,
                wasClean: closeEvent.wasClean
            });
        };
        this.ws.onmessage = (messageEvent) => {
            this.emit('message', {
                ...baseEvent,
                type: messageEvent.type,
                data: messageEvent.data
            });
        };
    }
    send(buf) {
        this.ws.send(buf);
    }
}
//# sourceMappingURL=wss.js.map